﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;

using Dapper;

namespace NoticeBoard.Models
{
    /// <summary>
    /// 게시 저장소
    /// </summary>
    public class NoticeRepository
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// SQL 연결
        /// </summary>
        private SqlConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - NoticeRepository()

        /// <summary>
        /// 생성자
        /// </summary>
        public NoticeRepository()
        {
            this.connection = new SqlConnection(ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 저장하기 - Save(notice, formType)

        /// <summary>
        /// 저장하기
        /// </summary>
        /// <param name="notice">게시글</param>
        /// <param name="formType">게시판 쓰기 폼 타입</param>
        /// <returns>처리 레코드 수</returns>
        public int Save(NoticeModel notice, BoardWriteFormType formType)
        {
            int recordCount = 0;

            DynamicParameters parameters = new DynamicParameters();

            parameters.Add("@Name"       , value : notice.Name       , dbType : DbType.String);
            parameters.Add("@MailAddress", value : notice.MailAddress, dbType : DbType.String);
            parameters.Add("@Title"      , value : notice.Title      , dbType : DbType.String);
            parameters.Add("@Content"    , value : notice.Content    , dbType : DbType.String);
            parameters.Add("@Password"   , value : notice.Password   , dbType : DbType.String);
            parameters.Add("@Encoding"   , value : notice.Encoding   , dbType : DbType.String);
            parameters.Add("@Homepage"   , value : notice.Homepage   , dbType : DbType.String);
            parameters.Add("@FileName"   , value : notice.FileName   , dbType : DbType.String);
            parameters.Add("@FileSize"   , value : notice.FileSize   , dbType : DbType.Int32 );

            switch(formType)
            {
                case BoardWriteFormType.Write :

                    parameters.Add("@WriteIP", value : notice.WriteIP, dbType : DbType.String);

                    recordCount = this.connection.Execute("WriteNotice", parameters, commandType : CommandType.StoredProcedure);

                    break;

                case BoardWriteFormType.Modify :

                    parameters.Add("@UpdateIP", value : notice.UpdateIP, dbType : DbType.String);
                    parameters.Add("@ID"      , value : notice.ID      , dbType : DbType.Int32 );

                    recordCount = this.connection.Execute("UpdateNotice", parameters, commandType: CommandType.StoredProcedure);

                    break;

                case BoardWriteFormType.Reply :

                    parameters.Add("@WriteIP" , value : notice.WriteIP , dbType : DbType.String);
                    parameters.Add("@ParentID", value : notice.ParentID, dbType : DbType.Int32 );

                    recordCount = this.connection.Execute("ReplyNotice", parameters, commandType: CommandType.StoredProcedure);

                    break;
            }

            return recordCount;
        }

        #endregion
        #region 추가하기 - Add(notice)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="notice">게시글</param>
        public void Add(NoticeModel notice)
        {
            try
            {
                Save(notice, BoardWriteFormType.Write);
            }
            catch(Exception exception)
            {
                throw new Exception(exception.Message);
            }
        }

        #endregion
        #region 수정하기 - Update(notice)

        /// <summary>
        /// 수정하기
        /// </summary>
        /// <param name="notice">게시글</param>
        /// <returns>처리 레코드 수</returns>
        public int Update(NoticeModel notice)
        {
            int recordCount = 0;

            try
            {
                recordCount = Save(notice, BoardWriteFormType.Modify);
            }
            catch(Exception exception)
            {
                throw new Exception(exception.Message);
            }

            return recordCount;
        }

        #endregion
        #region 답변하기 - Reply(notice)

        /// <summary>
        /// 답변하기
        /// </summary>
        /// <param name="notice">게시글</param>
        public void Reply(NoticeModel notice)
        {
            try
            {
                Save(notice, BoardWriteFormType.Reply);
            }
            catch(Exception exception)
            {
                throw new Exception(exception.Message);
            }
        }

        #endregion
        #region 리스트 구하기 - GetList(page)

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <param name="page">페이지</param>
        /// <returns>게시글 리스트</returns>
        public List<NoticeModel> GetList(int page)
        {
            try
            {
                DynamicParameters parameters = new DynamicParameters(new { Page = page });

                return this.connection.Query<NoticeModel>("ListNotice", parameters, commandType : CommandType.StoredProcedure).ToList();
            }
            catch(Exception exception)
            {
                throw new Exception(exception.Message);
            }
        }

        #endregion
        #region 검색하기 - Search(page, searchField, searchQuery)

        /// <summary>
        /// 검색하기
        /// </summary>
        /// <param name="page">페이지</param>
        /// <param name="searchField">검색 필드</param>
        /// <param name="searchQuery">검색 쿼리</param>
        /// <returns>게시글 리스트</returns>
        public List<NoticeModel> Search(int page, string searchField, string searchQuery)
        {
            DynamicParameters parameters = new DynamicParameters
            (
                new
                {
                    Page        = page,
                    SearchField = searchField,
                    SearchQuery = searchQuery
                }
            );

            return this.connection.Query<NoticeModel>("SearchNotice", parameters, commandType : CommandType.StoredProcedure).ToList();
        }

        #endregion
        #region 구하기 - Get(id)

        /// <summary>
        /// 구하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>게시글</returns>
        public NoticeModel Get(int id)
        {
            DynamicParameters parameters = new DynamicParameters(new { ID = id });

            return this.connection.Query<NoticeModel>("ViewNotice", parameters, commandType : CommandType.StoredProcedure).SingleOrDefault();
        }

        #endregion
        #region 카운트 구하기 - GetCount()

        /// <summary>
        /// 카운트 구하기
        /// </summary>
        /// <returns>카운트</returns>
        public int GetCount()
        {
            try
            {
                return this.connection.Query<int>("SELECT COUNT(*) FROM dbo.Notice").SingleOrDefault();
            }
            catch(Exception)
            {
                return -1;
            }
        }

        #endregion
        #region 카운트 구하기 - GetCount(searchField, searchQuery)

        /// <summary>
        /// 카운트 구하기
        /// </summary>
        /// <param name="searchField">검색 필드</param>
        /// <param name="searchQuery">검색 쿼리</param>
        /// <returns>카운트</returns>
        public int GetCount(string searchField, string searchQuery)
        {
            try
            {
                return this.connection.Query<int>
                (
                    "SearchNoticeCount",
                    new
                    {
                        SearchField = searchField,
                        SearchQuery = searchQuery
                    },
                    commandType : CommandType.StoredProcedure
                )
                .SingleOrDefault();
            }
            catch(Exception exception)
            {
                throw new Exception(exception.Message);
            }
        }

        #endregion
        #region 파일명 구하기 - GetFileName(id)

        /// <summary>
        /// 파일명 구하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>파일명</returns>
        public string GetFileName(int id)
        {
            return this.connection.Query<string>
            (
                "SELECT FileName FROM dbo.Notice WHERE ID = @ID",
                new { ID = id }
            ).SingleOrDefault();
        }

        #endregion
        #region 다운로드 카운트 수정하기 - UpdateDownloadCount(fileName)

        /// <summary>
        /// 다운로드 카운트 수정하기
        /// </summary>
        /// <param name="fileName">파일명</param>
        public void UpdateDownloadCount(string fileName)
        {
            this.connection.Execute
            (
                "UPDATE dbo.Notice SET DownloadCount = DownloadCount + 1 WHERE FileName = @FileName",
                new { FileName = fileName }
            );
        }

        #endregion
        #region 다운로드 카운트 수정하기 - UpdateDownloadCount(id)

        /// <summary>
        /// 다운로드 카운트 수정하기
        /// </summary>
        /// <param name="id">ID</param>
        public void UpdateDownloadCount(int id)
        {
            DynamicParameters parameters = new DynamicParameters(new { ID = id });

            this.connection.Execute
            (
                "UPDATE dbo.Notice SET DownloadCount = DownloadCount + 1 WHERE ID = @ID",
                parameters,
                commandType : CommandType.Text
            );
        }

        #endregion
        #region 삭제하기 - Delete(id, password)

        /// <summary>
        /// 삭제하기
        /// </summary>
        /// <returns>처리 레코드 수</returns>
        public int Delete(int id, string password)
        {
            return this.connection.Execute("DeleteNotice", new { ID = id, Password = password }, commandType : CommandType.StoredProcedure);
        }

        #endregion
    }
}